from pathlib import Path
from unittest.mock import Mock, patch

import polars as pl
import pytest

from blackfish.orca_parser import OrcaParser
from blackfish.parsing import Energies, ParsingError

# Sample content for mock files
SUCCESSFUL_CALC = """
Some calculation details...
TOTAL RUN TIME: 123.45 seconds
"""

FAILED_CALC = """
Some calculation details...
Error occurred...
"""

SUCCESSFUL_GEOM_OPT = """
Some calculation details...
Geometry convergence
...
HURRAY
...
TOTAL RUN TIME: 123.45 seconds
"""

FAILED_GEOM_OPT = """
Some calculation details...
Geometry convergence
...
Did not converge
...
TOTAL RUN TIME: 123.45 seconds
"""

@pytest.fixture
def mock_successful_file(tmp_path):
    file = tmp_path / "successful.out"
    file.write_text(SUCCESSFUL_CALC)
    return str(file)

@pytest.fixture
def mock_failed_file(tmp_path):
    file = tmp_path / "failed.out"
    file.write_text(FAILED_CALC)
    return str(file)

@pytest.fixture
def mock_successful_geom_opt(tmp_path):
    file = tmp_path / "successful_opt.out"
    file.write_text(SUCCESSFUL_GEOM_OPT)
    return str(file)

@pytest.fixture
def mock_failed_geom_opt(tmp_path):
    file = tmp_path / "failed_opt.out"
    file.write_text(FAILED_GEOM_OPT)
    return str(file)

class TestOrcaParser:
    def test_init_successful(self, mock_successful_file):
        parser = OrcaParser(mock_successful_file)
        assert isinstance(parser.file, Path)
        assert str(parser.file) == mock_successful_file

    def test_init_failed_calc(self, mock_failed_file):
        with pytest.raises(ParsingError, match="Calculation did not finish successfully"):
            OrcaParser(mock_failed_file)

    def test_successful_geom_opt(self, mock_successful_geom_opt):
        parser = OrcaParser(mock_successful_geom_opt)
        assert isinstance(parser.file, Path)

    def test_failed_geom_opt(self, mock_failed_geom_opt):
        with pytest.raises(ParsingError, match="Geometry optimization did not converge"):
            OrcaParser(mock_failed_geom_opt)

    @patch('blackfish.orca_parser.ir_spectrum')
    def test_get_ir_spectrum(self, mock_ir_spectrum, mock_successful_file):
        expected_df = pl.DataFrame({"col1": [1, 2, 3]})
        mock_ir_spectrum.return_value = expected_df

        parser = OrcaParser(mock_successful_file)
        result = parser.get_ir_spectrum()

        assert isinstance(result, pl.DataFrame)
        assert result.equals(expected_df)
        mock_ir_spectrum.assert_called_once_with(parser.file)

    @patch('blackfish.orca_parser.nacme')
    def test_get_nacme(self, mock_nacme, mock_successful_file):
        expected_df = pl.DataFrame({"col1": [1, 2, 3]})
        mock_nacme.return_value = expected_df

        parser = OrcaParser(mock_successful_file)
        result = parser.get_nacme()

        assert isinstance(result, pl.DataFrame)
        assert result.equals(expected_df)
        mock_nacme.assert_called_once_with(parser.file)

    @patch('blackfish.orca_parser.roots')
    def test_get_tddft_roots(self, mock_roots, mock_successful_file):
        expected_df = pl.DataFrame({"col1": [1, 2, 3]})
        mock_roots.return_value = expected_df

        parser = OrcaParser(mock_successful_file)
        result = parser.get_tddft_roots()

        assert isinstance(result, pl.DataFrame)
        assert result.equals(expected_df)
        mock_roots.assert_called_once_with(parser.file)

    @patch('blackfish.orca_parser.soc_absorption_spectrum')
    def test_get_tddft_soc_spectrum(self, mock_soc_spectrum, mock_successful_file):
        expected_df = pl.DataFrame({"col1": [1, 2, 3]})
        mock_soc_spectrum.return_value = expected_df

        parser = OrcaParser(mock_successful_file)
        result = parser.get_tddft_soc_spectrum()

        assert isinstance(result, pl.DataFrame)
        assert result.equals(expected_df)
        mock_soc_spectrum.assert_called_once_with(parser.file)

    @patch('blackfish.orca_parser.soc_states')
    def test_get_soc_states(self, mock_soc_states, mock_successful_file):
        expected_df = pl.DataFrame({"col1": [1, 2, 3]})
        mock_soc_states.return_value = expected_df

        parser = OrcaParser(mock_successful_file)
        result = parser.get_soc_states()

        assert isinstance(result, pl.DataFrame)
        assert result.equals(expected_df)
        mock_soc_states.assert_called_once_with(parser.file)

    @patch('blackfish.orca_parser.Energies')
    def test_get_energies(self, mock_energies, mock_successful_file):
        expected_energies = Mock(spec=Energies)
        mock_energies.return_value = expected_energies

        parser = OrcaParser(mock_successful_file)
        result = parser.get_energies()

        assert result == expected_energies
        mock_energies.assert_called_once_with(parser.file)

    @patch('blackfish.orca_parser.socme')
    def test_get_socme(self, mock_socme, mock_successful_file):
        expected_df = pl.DataFrame({"col1": [1, 2, 3]})
        mock_socme.return_value = expected_df

        parser = OrcaParser(mock_successful_file)
        result = parser.get_socme()

        assert isinstance(result, pl.DataFrame)
        assert result.equals(expected_df)
        mock_socme.assert_called_once_with(parser.file)
